package com.fr.plugin.third.party.jsdbacbj.config;

import com.fr.config.*;
import com.fr.config.holder.Conf;
import com.fr.config.holder.factory.Holders;

import java.util.HashMap;
import java.util.Map;

/**
 * 配置数据保存
 */
@Visualization(category = "登录集成配置")
public class CustomDataConfig extends DefaultConfiguration {
    public String getNameSpace() {
        return this.getClass().getName();
    }

    private static volatile CustomDataConfig config = null;

    public static CustomDataConfig getInstance() {
        if (config == null) {
            config = ConfigContext.getConfigInstance(CustomDataConfig.class);
        }
        return config;
    }


    private static volatile Map<String, String> URL_MAP = new HashMap<>();

    public synchronized static void addMapUrl(String key, String url) {
        URL_MAP.put(key, url);
    }

    public synchronized static String getMapUrl(String key) {
        if (!URL_MAP.containsKey(key)) {
            return "";
        }
        String url = URL_MAP.get(key);
        URL_MAP.remove(key);
        return url;
    }


    @Identifier(value = "clientId", name = "应用标识", description = "", status = Status.SHOW)
    private Conf<String> clientId = Holders.simple("");

    @Identifier(value = "clientSecret", name = "应用密钥", description = "", status = Status.SHOW)
    private Conf<String> clientSecret = Holders.simple("");

    @Identifier(value = "authorizeUrl", name = "请求用户授权地址", description = "", status = Status.SHOW)
    private Conf<String> authorizeUrl = Holders.simple("");

    @Identifier(value = "accessTokenUrl", name = "获取授权Token地址", description = "", status = Status.SHOW)
    private Conf<String> accessTokenUrl = Holders.simple("");

    @Identifier(value = "profileUrl", name = "获取用户信息地址", description = "", status = Status.SHOW)
    private Conf<String> profileUrl = Holders.simple("");

    @Identifier(value = "frUrl", name = "报表地址", description = "", status = Status.SHOW)
    private Conf<String> frUrl = Holders.simple("");

    @Identifier(value = "logoutUrl", name = "统一认证注销地址", description = "", status = Status.SHOW)
    private Conf<String> logoutUrl = Holders.simple("");




    @Identifier(value = "userDataUrl", name = "用户同步API地址", description = "", status = Status.HIDE)
    private Conf<String> userDataUrl = Holders.simple("");

    @Identifier(value = "userSystemCode", name = "用户同步系统编码(systemCode)", description = "", status = Status.HIDE)
    private Conf<String> userSystemCode = Holders.simple("");

    @Identifier(value = "userIntegrationKey", name = "用户同步集成密钥(integrationKey)", description = "", status = Status.HIDE)
    private Conf<String> userIntegrationKey = Holders.simple("");

    @Identifier(value = "userLoopTimes", name = "用户同步循环最大次数", description = "", status = Status.HIDE)
    private Conf<Integer> userLoopTimes = Holders.simple(10000);


    @Identifier(value = "dataSyncUrl", name = "待办发送接口地址", description = "", status = Status.HIDE)
    private Conf<String> dataSyncUrl = Holders.simple("");

    @Identifier(value = "dataUsername", name = "待办发送接口用户名", description = "", status = Status.HIDE)
    private Conf<String> dataUsername = Holders.simple("");

    @Identifier(value = "dataPassword", name = "待办发送接口密码", description = "", status = Status.HIDE)
    private Conf<String> dataPassword = Holders.simple("");

    @Identifier(value = "appName", name = "待办发送来源", description = "", status = Status.HIDE)
    private Conf<String> appName = Holders.simple("");

    @Identifier(value = "modelName", name = "待办发送模块名", description = "", status = Status.HIDE)
    private Conf<String> modelName = Holders.simple("");

    @Identifier(value = "docCreator", name = "待办发送待办创建者", description = "", status = Status.HIDE)
    private Conf<String> docCreator = Holders.simple("");



    @Identifier(value = "loginTypeNameParameter", name = "登录类型参数名称", description = "", status = Status.HIDE)
    private Conf<String> loginTypeNameParameter = Holders.simple("loginType");

    @Identifier(value = "loginTypeValue", name = "登录类型值", description = "", status = Status.HIDE)
    private Conf<String> loginTypeValue = Holders.simple("oauth");

    public String getLoginTypeNameParameter() {
        return loginTypeNameParameter.get();
    }

    public void setLoginTypeNameParameter(String loginTypeNameParameter) {
        this.loginTypeNameParameter.set(loginTypeNameParameter);
    }

    public String getLoginTypeValue() {
        return loginTypeValue.get();
    }

    public void setLoginTypeValue(String loginTypeValue) {
        this.loginTypeValue.set(loginTypeValue);
    }

    public Integer getUserLoopTimes() {
        return userLoopTimes.get();
    }

    public void setUserLoopTimes(Integer userLoopTimes) {
        this.userLoopTimes.set(userLoopTimes);
    }

    public String getUserDataUrl() {
        return userDataUrl.get();
    }

    public void setUserDataUrl(String userDataUrl) {
        this.userDataUrl.set(userDataUrl);
    }

    public String getUserSystemCode() {
        return userSystemCode.get();
    }

    public void setUserSystemCode(String userSystemCode) {
        this.userSystemCode.set(userSystemCode);
    }

    public String getUserIntegrationKey() {
        return userIntegrationKey.get();
    }

    public void setUserIntegrationKey(String userIntegrationKey) {
        this.userIntegrationKey.set(userIntegrationKey);
    }

    public String getDataSyncUrl() {
        return dataSyncUrl.get();
    }

    public void setDataSyncUrl(String dataSyncUrl) {
        this.dataSyncUrl.set(dataSyncUrl);
    }

    public String getDataUsername() {
        return dataUsername.get();
    }

    public void setDataUsername(String dataUsername) {
        this.dataUsername.set(dataUsername);
    }

    public String getDataPassword() {
        return dataPassword.get();
    }

    public void setDataPassword(String dataPassword) {
        this.dataPassword.set(dataPassword);
    }

    public String getAppName() {
        return appName.get();
    }

    public void setAppName(String appName) {
        this.appName.set(appName);
    }

    public String getModelName() {
        return modelName.get();
    }

    public void setModelName(String modelName) {
        this.modelName.set(modelName);
    }

    public String getDocCreator() {
        return docCreator.get();
    }

    public void setDocCreator(String docCreator) {
        this.docCreator.set(docCreator);
    }

    public String getClientId() {
        return clientId.get();
    }

    public void setClientId(String clientId) {
        this.clientId.set(clientId);
    }

    public String getClientSecret() {
        return clientSecret.get();
    }

    public void setClientSecret(String clientSecret) {
        this.clientSecret.set(clientSecret);
    }

    public String getAuthorizeUrl() {
        return authorizeUrl.get();
    }

    public void setAuthorizeUrl(String authorizeUrl) {
        this.authorizeUrl.set(authorizeUrl);
    }

    public String getAccessTokenUrl() {
        return accessTokenUrl.get();
    }

    public void setAccessTokenUrl(String accessTokenUrl) {
        this.accessTokenUrl.set(accessTokenUrl);
    }

    public String getProfileUrl() {
        return profileUrl.get();
    }

    public void setProfileUrl(String profileUrl) {
        this.profileUrl.set(profileUrl);
    }

    public String getFrUrl() {
        return frUrl.get();
    }

    public void setFrUrl(String frUrl) {
        this.frUrl.set(frUrl);
    }

    public String getLogoutUrl() {
        return logoutUrl.get();
    }

    public void setLogoutUrl(String logoutUrl) {
        this.logoutUrl.set(logoutUrl);
    }

    @Override
    public Object clone() throws CloneNotSupportedException {
        CustomDataConfig cloned = (CustomDataConfig) super.clone();
        cloned.dataSyncUrl = (Conf<String>) dataSyncUrl.clone();
        cloned.dataUsername = (Conf<String>) dataUsername.clone();
        cloned.dataPassword = (Conf<String>) dataPassword.clone();
        cloned.appName = (Conf<String>) appName.clone();
        cloned.modelName = (Conf<String>) modelName.clone();
        cloned.docCreator = (Conf<String>) docCreator.clone();
        cloned.userDataUrl = (Conf<String>) userDataUrl.clone();
        cloned.userSystemCode = (Conf<String>) userSystemCode.clone();
        cloned.userIntegrationKey = (Conf<String>) userIntegrationKey.clone();
        cloned.userLoopTimes = (Conf<Integer>) userLoopTimes.clone();
        cloned.loginTypeNameParameter = (Conf<String>) loginTypeNameParameter.clone();
        cloned.loginTypeValue = (Conf<String>) loginTypeValue.clone();
        cloned.clientId = (Conf<String>) clientId.clone();
        cloned.clientSecret = (Conf<String>) clientSecret.clone();
        cloned.authorizeUrl = (Conf<String>) authorizeUrl.clone();
        cloned.accessTokenUrl = (Conf<String>) accessTokenUrl.clone();
        cloned.profileUrl = (Conf<String>) profileUrl.clone();
        cloned.frUrl = (Conf<String>) frUrl.clone();
        cloned.logoutUrl = (Conf<String>) logoutUrl.clone();
        return cloned;
    }
}
