package com.fr.plugin.third.party.jsdbacbj.http;

import com.fanruan.api.log.LogKit;
import com.fanruan.api.util.StringKit;
import com.fr.decision.authority.data.User;
import com.fr.decision.fun.impl.AbstractGlobalRequestFilterProvider;
import com.fr.decision.webservice.v10.login.LoginService;
import com.fr.decision.webservice.v10.user.UserService;
import com.fr.general.ComparatorUtils;
import com.fr.json.JSONArray;
import com.fr.json.JSONObject;
import com.fr.plugin.third.party.jsdbacbj.Utils;
import com.fr.plugin.third.party.jsdbacbj.config.CustomDataConfig;
import com.fr.third.org.apache.commons.codec.digest.DigestUtils;
import com.fr.third.org.apache.http.HttpEntity;
import com.fr.third.org.apache.http.HttpStatus;
import com.fr.third.org.apache.http.NameValuePair;
import com.fr.third.org.apache.http.client.config.RequestConfig;
import com.fr.third.org.apache.http.client.entity.UrlEncodedFormEntity;
import com.fr.third.org.apache.http.client.methods.CloseableHttpResponse;
import com.fr.third.org.apache.http.client.methods.HttpGet;
import com.fr.third.org.apache.http.client.methods.HttpPost;
import com.fr.third.org.apache.http.impl.client.CloseableHttpClient;
import com.fr.third.org.apache.http.message.BasicNameValuePair;
import com.fr.third.org.apache.http.util.EntityUtils;
import com.fr.web.utils.WebUtils;

import javax.servlet.FilterChain;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.*;


public class SessionGlobalRequestFilterProvider extends AbstractGlobalRequestFilterProvider {
    private static String DEFAULT_USER_AGENT = "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/83.0.4103.61 Safari/537.36";

    @Override
    public String filterName() {
        return "com.fr.plugin.third.party.jsdbacbj";
    }

    @Override
    public String[] urlPatterns() {
        return new String[]{"/decision", "/decision/*"};
    }

    @Override
    public void doFilter(HttpServletRequest req, HttpServletResponse res, FilterChain filterChain) {
        try {
            String fullUrl = req.getRequestURL().toString();
            String queryUrl = req.getQueryString();
            if ((queryUrl == null) || "null".equalsIgnoreCase(queryUrl)) {
                queryUrl = "";
            } else {
                queryUrl = "?" + queryUrl;
            }

            String fullUrl1 = fullUrl + queryUrl;
            String method = req.getMethod();
            LogKit.info("登录集成,记录访问地址：" + method + " " + fullUrl1);
            if (!"GET".equalsIgnoreCase(method)) {
                filterChain.doFilter(req, res);
                return;
            }

            if (isNoAuthRequest(req)) {
                filterChain.doFilter(req, res);
                return;
            }

            if (fullUrl.indexOf("/remote/") >= 0) {
                filterChain.doFilter(req, res);
                return;
            }

            if (fullUrl.indexOf("terminal=H5") >= 0) {
                filterChain.doFilter(req, res);
                return;
            }

            if (fullUrl.indexOf("__device__=") >= 0) {
                filterChain.doFilter(req, res);
                return;
            }

            if (fullUrl.indexOf("/weixin/") >= 0) {
                filterChain.doFilter(req, res);
                return;
            }


            if (fullUrl.indexOf("/dingtalk/") >= 0) {
                filterChain.doFilter(req, res);
                return;
            }

            if (isLoginRequest(req)) {
                LogKit.info("登录集成,登录页,访问地址：" + method + " " + fullUrl1);
                filterChain.doFilter(req, res);
                return;
            }
            //boolean option = isLogged(req);
            boolean option = LoginService.getInstance().isLogged(req);
            if (option && (!isOauthCodeRequest(req)) && (!isAllowLoginWithParameter(req))) {
                LogKit.info("登录集成,标记a,访问地址：" + method + " " + fullUrl1);
                filterChain.doFilter(req, res);
                return;
            }

            String state, reqUrl;
            if (isAllowLoginWithParameter(req) || ((!option) && (!isOauthCodeRequest(req)) && isReportRequest(req))) {
                LogKit.info("登录集成,标记b,访问地址：" + method + " " + fullUrl1);
                reqUrl = getRequestUrl(req);
                state = Utils.getUuid();
                CustomDataConfig.getInstance().addMapUrl(state, reqUrl);
                String tempUrl = Utils.getFullAuthorizeUrl(state);
                LogKit.info("登录集成,请求认证地址：" + tempUrl);
                sendRedirect(res, tempUrl);
                return;
            }

            String loginUsername = getOauthLoginUsername(req);
            if (StringKit.isEmpty(loginUsername)) {
                filterChain.doFilter(req, res);
                return;
            }
            LogKit.info("登录集成,用户名:" + loginUsername);

            User user = UserService.getInstance().getUserByUserName(loginUsername);
            boolean tipsOption = false;
            String tipsContent = "";
            if (user == null) {
                tipsOption = true;
                LogKit.info("登录集成,用户名:" + loginUsername + "在报表平台不存在");
                tipsContent = "在报表服务器上不存在";
            } else if (!user.isEnable()) {
                tipsOption = true;
                LogKit.info("登录集成,用户名:" + loginUsername + "在报表平台上被禁用");
                tipsContent = "在报表平台上被禁用";
            }

            if (tipsOption) {
                String jumpContent = "<!doctype html>\n" +
                        "<head>\n" +
                        "    <meta charset=\"utf-8\" />\n" +
                        "    <title>提示</title>\n" +
                        "</head>\n" +
                        "<body>\n" +
                        "    <script>\n" +
                        "        var t = 10;\n" +
                        "        var referI =  setInterval(\"refer()\", 1000);\n" +
                        "        function refer() {\n" +
                        "            document.getElementById('show').innerHTML = \"用户:" + loginUsername + tipsContent + ",请联系管理员!<br>\" + t + \"秒后跳转到报表首页\"; \n" +
                        "            t--;\n" +
                        "            if (t <= 0) {\n" +
                        "                clearInterval(referI);\n" +
                        "                window.location = \"" + Utils.getLogoutUrl() + "\";\n" +
                        "setTimeout(function(){\n" +
                        "                window.location = \"" + Utils.getFullAuthorizeUrlWithState() + "\";\n" +
                        "}, 1000);" +
                        "            }\n" +
                        "        }\n" +
                        "    </script>\n" +
                        "    <div style=\"width: 100%;height:200px;  line-height: 200px;font-size:30px;vertical-align:middle;text-align:center\">\n" +
                        "        <span id=\"show\"></span>\n" +
                        "    </div>\n" +
                        "</body>\n" +
                        "</html>";

                jumpContent = "<!doctype html>\n" +
                        "<head>\n" +
                        "    <meta charset=\"utf-8\" />\n" +
                        "    <title>提示</title>\n" +
                        "\t\n" +
                        "<script type=\"text/javascript\">\n" +
                        "window.onload=function(){\n" +
                        "            alert(\"用户:" + loginUsername + tipsContent + ",请联系管理员!\");\n" +
                        "            window.location = \"" + Utils.getLogoutUrl() + "\";\n" +
                        "           setTimeout(function(){\n" +
                        "                window.location = \"" + Utils.getFullAuthorizeUrlWithState() + "\";\n" +
                        "           }, 1000); \n" +
                        "}\n" +
                        "</script>\t\n" +
                        "</head>\n" +
                        "<body>\n" +
                        "    <div style=\"width: 100%;height:200px;  line-height: 200px;font-size:30px;vertical-align:middle;text-align:center\">\n" +
                        "        <span id=\"show\"></span>\n" +
                        "    </div>\n" +
                        "</body>\n" +
                        "</html>";
                res.setContentType("text/html;charset=UTF-8");
                WebUtils.printAsString(res, jumpContent);
                res.setStatus(200);
                return;
            }

            loginUsername = user.getUserName();
            LogKit.info("登录集成,报表平台用户名:" + loginUsername);

            String loginToken = LoginService.getInstance().login(req, res, loginUsername);
            req.setAttribute("fine_auth_token", loginToken);

            reqUrl = getRealUrl(req);
            if (StringKit.isNotEmpty(reqUrl)) {
                sendRedirect(res, reqUrl);
                return;
            }

            filterChain.doFilter(req, res);
        } catch (Exception e) {
            LogKit.error("登录集成出错," + e.getMessage(), e);
        }
    }

    private String getRealUrl(HttpServletRequest req) {
        if (req == null) {
            return "";
        }
        String state = WebUtils.getHTTPRequestParameter(req, "state");
        if (StringKit.isEmpty(state)) {
            return "";
        }
        String url = CustomDataConfig.getInstance().getMapUrl(state);
        return url;
    }


    private String getOauthLoginUsername(HttpServletRequest req) throws IOException {
        if (req == null) {
            return "";
        }
        String oAuthCode = WebUtils.getHTTPRequestParameter(req, "code");
        if (StringKit.isEmpty(oAuthCode)) {
            return "";
        }
        LogKit.info("登录集成,OAuth Code:" + oAuthCode);

        RequestConfig requestConfig = RequestConfig.custom()
                .setConnectionRequestTimeout(10000)
                .setSocketTimeout(10000) // 服务端相应超时
                .setConnectTimeout(10000) // 建立socket链接超时时间
                .build();

        //获取授权Token
       
        String accessTokenUrl = CustomDataConfig.getInstance().getAccessTokenUrl();
        LogKit.info("登录集成,获取授权Token地址:" + accessTokenUrl);
        HttpPost httpPost = new HttpPost(accessTokenUrl);
        httpPost.addHeader("User-Agent", DEFAULT_USER_AGENT);


        List<NameValuePair> params = new ArrayList<NameValuePair>();
        NameValuePair clientIdNameValuePair = new BasicNameValuePair("client_id", CustomDataConfig.getInstance().getClientId());
        NameValuePair clientSecretNameValuePair = new BasicNameValuePair("client_secret", CustomDataConfig.getInstance().getClientSecret());
        NameValuePair redirectUriNameValuePair = new BasicNameValuePair("redirect_uri", CustomDataConfig.getInstance().getFrUrl());
        NameValuePair codeNameValuePair = new BasicNameValuePair("code", oAuthCode);
        NameValuePair grantTypeNameValuePair = new BasicNameValuePair("grant_type", "authorization_code");
        
        params.add(clientIdNameValuePair);
        params.add(clientSecretNameValuePair);
        
        params.add(codeNameValuePair);
        params.add(grantTypeNameValuePair);
        
        httpPost.setEntity(new UrlEncodedFormEntity(params, "UTF-8"));
        httpPost.setHeader("Content-type", "application/x-www-form-urlencoded");


        httpPost.setConfig(requestConfig);
        CloseableHttpClient httpClient = Utils.createHttpClient(accessTokenUrl);
        CloseableHttpResponse response = httpClient.execute(httpPost);
        int statusCode = response.getStatusLine().getStatusCode();
        if (statusCode != HttpStatus.SC_OK) {
            response.close();
            httpClient.close();
            LogKit.info("登录集成,获取授权Token请求出错,http status:" + statusCode);
            return "";
        }

        HttpEntity httpEntity = response.getEntity();
        if (httpEntity == null) {
            response.close();
            httpClient.close();
            LogKit.info("登录集成,获取授权Token请求出错,http响应内容为空");
            return "";
        }
        String responseContent = EntityUtils.toString(httpEntity, "UTF-8");
        response.close();
        if (StringKit.isEmpty(responseContent)) {
            httpClient.close();
            LogKit.info("登录集成,获取授权Token请求出错,http响应内容为空1");
            return "";
        }
        LogKit.info("登录集成,获取授权Token请求,http响应内容\n" + responseContent);

        String accessToken = getAccessToken(responseContent);
        if (StringKit.isEmpty(accessToken)) {
            httpClient.close();
            LogKit.info("登录集成,获取授权Token请求出错,授权Token为空");
            return "";
        }
        LogKit.info("登录集成,授权Token:" + accessToken);

        
        String userUrl = CustomDataConfig.getInstance().getProfileUrl() + "?access_token=" + accessToken + "&client_id=" + CustomDataConfig.getInstance().getClientId();
        HttpGet httpGet = new HttpGet(userUrl);
        
        httpGet.setConfig(requestConfig);
        response = httpClient.execute(httpGet);
        statusCode = response.getStatusLine().getStatusCode();
        if (statusCode != HttpStatus.SC_OK) {
            response.close();
            httpClient.close();
            LogKit.info("登录集成,获取用户信息请求出错,http status:" + statusCode);
            return "";
        }

        httpEntity = response.getEntity();
        if (httpEntity == null) {
            response.close();
            httpClient.close();
            LogKit.info("登录集成,获取用户信息请求出错,http响应内容为空");
            return "";
        }
        responseContent = EntityUtils.toString(httpEntity, "UTF-8");
        response.close();
        httpClient.close();
        if (StringKit.isEmpty(responseContent)) {
            LogKit.info("登录集成,获取用户信息请求出错,http响应内容为空1");
            return "";
        }

        LogKit.info("登录集成,获取用户信息请求,http响应内容\n" + responseContent);
        String uid = getUserId(responseContent);
        if (StringKit.isEmpty(uid)) {
            LogKit.info("登录集成,,获取用户信息请求出错,uid为空");
            return "";
        }
        LogKit.info("登录集成,uid:" + uid);

        return uid;
    }


    public static String getSign(Map<String, String> params, String secret) {
        String sign = "";
        StringBuilder sb = new StringBuilder();
        //排序
        Set<String> keyset = params.keySet();
        TreeSet<String> sortSet = new TreeSet<String>();
        sortSet.addAll(keyset);
        Iterator<String> it = sortSet.iterator();
        //加密字符串
        while (it.hasNext()) {
            String key = it.next();
            String value = params.get(key);
            sb.append(key).append(value);
        }
        sb.append("appkey").append(secret);
        String md5Str = sb.toString();
        LogKit.info("登录集成,获取授权Token,sign 待加密字符串:" + md5Str);
        try {
            sign = DigestUtils.md5Hex(md5Str).toUpperCase();
        } catch (Exception e) {
        }
        return sign;
    }


    private String getAccessToken(String content) {
        if (StringKit.isEmpty(content)) {
            return "";
        }

        JSONObject contentJson = new JSONObject(content);
        String token = contentJson.getString("access_token");
        return token;
    }

    private String getUserId(String content) {
        if (StringKit.isEmpty(content)) {
            return "";
        }

        String loginName;
        JSONObject contentJson = new JSONObject(content);
        if (contentJson.containsKey("spRoleList")) {
            JSONArray jsonArray = contentJson.getJSONArray("spRoleList");
            if ((jsonArray != null) && (jsonArray.size() >= 1)) {
                return jsonArray.getString(0);
            }
        }
        loginName = contentJson.getString("loginName");
        return loginName;
    }


    public synchronized static String getSysTime() {
        SimpleDateFormat format = new SimpleDateFormat("yyyyMMdd");
        Date date = new Date();
        String nowData = format.format(date);
        return nowData;
    }

    private void sendRedirect(HttpServletResponse res, String url) {
        res.setStatus(HttpServletResponse.SC_MOVED_TEMPORARILY);
        res.setHeader("Location", url);
    }

    private String getRequestUrl(HttpServletRequest req) {
        String fullUrl = req.getRequestURL().toString();
        fullUrl = getRealUrl(fullUrl);
        Map<String, String[]> paraMap = req.getParameterMap();
        String paraName;
        String[] paraValues;
        String loginTypeParaName = CustomDataConfig.getInstance().getLoginTypeNameParameter();
        String queryStr = "";
        for (Map.Entry<String, String[]> entry : paraMap.entrySet()) {
            paraName = entry.getKey();
            if (ComparatorUtils.equals(paraName, loginTypeParaName)) {
                continue;
            }
            if (ComparatorUtils.equals(paraName, "code")) {
                continue;
            }
            paraValues = entry.getValue();
            queryStr = addParaToQuery(queryStr, paraName, paraValues);
        }
        if (StringKit.isEmpty(queryStr)) {
            return fullUrl;
        }
        fullUrl = fullUrl + "?" + queryStr;
        return fullUrl;
    }

    private String getRealUrl(String url) {
        if (StringKit.isEmpty(url)) {
            return url;
        }
        int index = url.indexOf("/decision");
        if (index < 0) {
            return url;
        }
        String tempUrl = CustomDataConfig.getInstance().getFrUrl() + url.substring(index + "/decision".length());
        return tempUrl;
    }

    private String addParaToQuery(String query, String paraName, String[] paraValues) {
        if (StringKit.isEmpty(paraName)) {
            return query;
        }
        String fullQuery = query;
        if ((paraValues == null) || (paraValues.length <= 0)) {
            if (StringKit.isNotEmpty(fullQuery)) {
                fullQuery = fullQuery + "&";
            }
            fullQuery = paraName + "=";
            return fullQuery;
        }
        for (int i = 0, max = paraValues.length - 1; i <= max; i++) {
            if (StringKit.isNotEmpty(fullQuery)) {
                fullQuery = fullQuery + "&";
            }
            fullQuery = fullQuery + paraName + "=" + paraValues[i];
        }
        return fullQuery;
    }


    private boolean isAllowLoginWithParameter(HttpServletRequest req) {
        if (req == null) {
            return false;
        }
        String loginTypeNameParameter = CustomDataConfig.getInstance().getLoginTypeNameParameter();
        String loginTypeConfigValue = CustomDataConfig.getInstance().getLoginTypeValue();
        if (StringKit.isEmpty(loginTypeNameParameter) || StringKit.isEmpty(loginTypeConfigValue)) {
            return false;
        }
        String loginTypeValue = WebUtils.getHTTPRequestParameter(req, loginTypeNameParameter);
        return ComparatorUtils.equalsIgnoreCase(loginTypeConfigValue, loginTypeValue);
    }


    private boolean isReportRequest(HttpServletRequest req) {
        if (req == null) {
            return false;
        }
        if (!"GET".equalsIgnoreCase(req.getMethod())) {
            return false;
        }
        String url = req.getRequestURL().toString();
        if (url.endsWith("/decision") || url.endsWith("/decision/")) {
            return true;
        }
        if ((url.indexOf("/decision/") >= 0) && (url.indexOf("/entry/access/") >= 0)) {
            return true;
        }
        String viewlet = WebUtils.getHTTPRequestParameter(req, "viewlet");
        if ((url.indexOf("/decision/view/report") >= 0) && (StringKit.isNotEmpty(viewlet))) {
            return true;
        }
        if ((url.indexOf("/decision/view/form") >= 0) && (StringKit.isNotEmpty(viewlet))) {
            return true;
        }
        return false;
    }

    private boolean isOauthCodeRequest(HttpServletRequest req) throws IOException {
        if (req == null) {
            return false;
        }
        if (!"GET".equalsIgnoreCase(req.getMethod())) {
            return false;
        }
        String oAuthCode = WebUtils.getHTTPRequestParameter(req, "code");
        if (StringKit.isNotEmpty(oAuthCode)) {
            return true;
        }
        return false;
    }

    private boolean isLoginRequest(HttpServletRequest req) {
        if (req == null) {
            return false;
        }
        if (!"GET".equalsIgnoreCase(req.getMethod())) {
            return false;
        }
        String url = req.getRequestURL().toString();
        if (url.endsWith("/decision/login") || url.endsWith("/decision/login/")) {
            return true;
        }
        return false;
    }

    private boolean isNoAuthRequest(HttpServletRequest req) throws IOException {
        if (req == null) {
            return false;
        }
        if (!"GET".equalsIgnoreCase(req.getMethod())) {
            return false;
        }
        String oAuthCode = WebUtils.getHTTPRequestParameter(req, "loginType");
        if (StringKit.equalsIgnoreCase("noauth", oAuthCode)) {
            return true;
        }
        return false;
    }

}
